@@ -20,6 +20,7 @@
 from collections import defaultdict
 from collections import namedtuple
 from collections import OrderedDict
+import functools
 import html.entities
 import logging
 import mimetypes
@@ -38,11 +39,13 @@ from turberfield.dialogue.directives import Property as PropertyDirective
 from turberfield.dialogue.directives import Memory as MemoryDirective
 from turberfield.utils.assembly import Assembly
 from turberfield.utils.misc import group_by_type
+from turberfield.utils.logger import LogManager
 
 import pkg_resources
 import docutils
 from docutils.frontend import Values
 from docutils.nodes import block_quote
+from docutils.nodes import citation
 from docutils.nodes import field_body
 from docutils.nodes import list_item
 
@@ -55,14 +58,38 @@ class Model(docutils.nodes.GenericNodeVisitor):
 
     """
 
-    Shot = namedtuple("Shot", ["name", "scene", "items"])
-    Property = namedtuple("Property", ["entity", "object", "attr", "val"])
-    Audio = namedtuple("Audio", ["package", "resource", "offset", "duration", "loop"])
-    Still = namedtuple("Still", list(Audio._fields) + ["label", "width", "height"])
-    Video = namedtuple("Video", list(Still._fields) + ["poster", "url"])
-    Memory = namedtuple("Memory", ["subject", "object", "state", "text", "html"])
-    Line = namedtuple("Line", ["persona", "text", "html"])
-    Condition = namedtuple("Condition", ["object", "format", "regex", "value"])
+    Shot = namedtuple("Shot", ["name", "scene", "items", "path", "line_nr"], defaults=(None, None))
+    Property = namedtuple(
+        "Property",
+        ["entity", "object", "attr", "val", "path", "line_nr"],
+        defaults=(None, None)
+    )
+    Audio = namedtuple(
+        "Audio",
+        ["package", "resource", "offset", "duration", "loop", "path", "line_nr"],
+        defaults=(None, None)
+    )
+    Still = namedtuple(
+        "Still",
+        list(Audio._fields[:-2]) + ["label", "width", "height", "path", "line_nr"],
+        defaults=(None, None)
+    )
+    Video = namedtuple(
+        "Video",
+        list(Still._fields[:-2]) + ["poster", "url", "path", "line_nr"],
+        defaults=(None, None)
+    )
+    Memory = namedtuple(
+        "Memory",
+        ["subject", "object", "state", "text", "html", "path", "line_nr"],
+        defaults=(None, None)
+    )
+    Line = namedtuple("Line", ["persona", "text", "html", "path", "line_nr"], defaults=(None, None))
+    Condition = namedtuple(
+        "Condition",
+        ["object", "format", "regex", "value", "path", "line_nr"],
+        defaults=(None, None)
+    )
 
     def __init__(self, fP, document):
         super().__init__(document)
@@ -74,9 +101,15 @@ class Model(docutils.nodes.GenericNodeVisitor):
                 FXDirective.Cue, ConditionDirective.Evaluation
             )
         )
-        self.log = logging.getLogger(
-            "turberfield.dialogue.{0}".format(os.path.basename(self.fP))
+        self.log_manager = LogManager()
+        self.log = self.log_manager.clone(
+            self.log_manager.get_logger("main"), "turberfield.dialogue.model"
         )
+        self.log.frame = [
+            "{now}", "{level.name:>8}", "{logger.name}",
+            "{1[path]}", "{1[line_nr]:>5}", " {0}", " {token}"
+        ]
+
         self.section_level = 0
         self.scenes = [None]
         self.shots = [Model.Shot(None, None, [])]
@@ -96,7 +129,7 @@ class Model(docutils.nodes.GenericNodeVisitor):
             for item in shot.items:
                 yield shot, item
 
-    def close_shot(self):
+    def close_shot(self, line_nr=None):
         if self.memory:
             self.shots[-1].items.append(
                 self.memory._replace(text="".join(self.text), html="\n".join(self.html))
@@ -104,7 +137,7 @@ class Model(docutils.nodes.GenericNodeVisitor):
             self.memory = None
         elif self.text:
             self.shots[-1].items.append(
-                Model.Line(self.speaker, "".join(self.text), "\n".join(self.html))
+                Model.Line(self.speaker, "".join(self.text), "\n".join(self.html), self.fP, line_nr)
             )
             self.text.clear()
             self.html.clear()
@@ -116,7 +149,7 @@ class Model(docutils.nodes.GenericNodeVisitor):
             if ref and ref.lower() in entity.attributes["names"]),
             None)
 
-    def substitute_property(self, matchObj):
+    def substitute_property(self, matchObj, line=None):
         try:
             defn = self.document.substitution_defs[matchObj.group(1)]
             getter = next(
@@ -127,12 +160,16 @@ class Model(docutils.nodes.GenericNodeVisitor):
             entity = self.get_entity(ref)
             rv = str(operator.attrgetter(attr)(entity.persona)).strip()
         except (AttributeError, KeyError, IndexError, StopIteration) as e:
-            self.log.warning("Argument has bad substitution ref {0}".format(matchObj.group(1)))
+            self.log.warning(
+                "Argument has bad substitution reference",
+                {"path": self.fP, "line_nr": line},
+                token=matchObj.group(1)
+            )
             rv = ""
         return rv
 
     def default_visit(self, node):
-        self.log.debug(node)
+        self.log.debug(node, {"path": self.fP, "line_nr": node.line})
 
     def default_departure(self, node):
         pass
@@ -145,19 +182,28 @@ class Model(docutils.nodes.GenericNodeVisitor):
 
     def depart_bullet_list(self, node):
         self.html.append("</ul>")
-        self.close_shot()
+        self.close_shot(node.line)
 
     def visit_citation_reference(self, node):
         entity = self.get_entity(node.attributes["refname"])
-        self.speaker = entity and entity.persona
+        try:
+            self.speaker = entity.persona
+        except AttributeError:
+            self.log.warning(
+                "Reference to entity with no persona",
+                {"path": self.fP, "line_nr": node.parent.line},
+                node=node, entity=entity, token=node.rawsource
+            )
+            self.speaker = node.attributes["refname"]
 
     def visit_Cue(self, node):
         subref_re = re.compile("\|(\w+)\|")
+        subber = functools.partial(self.substitute_property, line=node.parent.line)
         pkg = node["arguments"][0]
-        rsrc = subref_re.sub(self.substitute_property, node["arguments"][1])
+        rsrc = subref_re.sub(subber, node["arguments"][1])
         offset = node["options"].get("offset")
         duration = node["options"].get("duration")
-        label = subref_re.sub(self.substitute_property, node["options"].get("label", ""))
+        label = subref_re.sub(subber, node["options"].get("label", ""))
         loop = node["options"].get("loop")
         width = node["options"].get("width")
         height = node["options"].get("height")
@@ -165,13 +211,14 @@ class Model(docutils.nodes.GenericNodeVisitor):
         item = None
         try:
             if typ.startswith("audio"):
-                item = Model.Audio(pkg, rsrc, offset, duration, loop)
+                item = Model.Audio(pkg, rsrc, offset, duration, loop, self.fP, node.line)
             elif typ.startswith("image"):
-                item = Model.Still(pkg, rsrc, offset, duration, loop, label, width, height)
+                item = Model.Still(pkg, rsrc, offset, duration, loop, label, width, height, self.fP, node.line)
             elif typ.startswith("video"):
                 item = Model.Video(
                     pkg, rsrc, offset, duration, loop, label, width, height,
-                    *(node["options"].get(i, None) for i in ["poster", "url"])
+                    *(node["options"].get(i, None) for i in ["poster", "url"]),
+                    path=self.fP, line_nr=node.line
                 )
         except AttributeError:
             pass
@@ -184,7 +231,7 @@ class Model(docutils.nodes.GenericNodeVisitor):
         subj = self.get_entity(node["options"].get("subject"))
         obj = self.get_entity(node["options"].get("object"))
         self.memory = Model.Memory(
-            subj and subj.persona, obj and obj.persona, state, None, None
+            subj and subj.persona, obj and obj.persona, state, None, None, self.fP, node.line
         )
 
     def visit_emphasis(self, node):
@@ -204,17 +251,22 @@ class Model(docutils.nodes.GenericNodeVisitor):
                 regex = re.compile(pattern)
                 value = pattern
             except Exception as e:
-                self.log.warning("Condition regex error {0} {1}".format(e, pattern))
+                self.log.warning(
+                    "Condition regex error",
+                    {"path": self.fP, "line_nr": node.line},
+                    token=pattern, exception=e
+                )
 
         if not regex:
-            s = re.compile("\|(\w+)\|").sub(self.substitute_property, pattern)
+            subber = functools.partial(self.substitute_property, line=node.line)
+            s = re.compile("\|(\w+)\|").sub(subber, pattern)
             try:
                 value = int(s) if s.isdigit() else node.string_import(s)
             except ValueError:
                 value = s
 
         self.shots[-1].items.append(
-            Model.Condition(entity.persona, format_, regex, value)
+            Model.Condition(entity.persona, format_, regex, value, self.fP, node.line)
         )
 
     def depart_field_name(self, node):
@@ -240,14 +292,14 @@ class Model(docutils.nodes.GenericNodeVisitor):
         ))
 
     def visit_paragraph(self, node):
-        if self.shots and not isinstance(node.parent, (field_body, list_item)):
+        if self.shots and not isinstance(node.parent, (citation, field_body, list_item)):
             self.text = []
             self.html = ["<p>"]
 
     def depart_paragraph(self, node):
-        if self.shots and not isinstance(node.parent, (field_body, list_item)):
+        if self.shots and not isinstance(node.parent, (citation, field_body, list_item)):
             self.html.append("</p>\n")
-            self.close_shot()
+            self.close_shot(node.line)
 
     def depart_raw(self, node):
         if "html" in node.attributes["format"] and self.shots:
@@ -279,13 +331,16 @@ class Model(docutils.nodes.GenericNodeVisitor):
         ref, attr = node["arguments"][0].split(".")
         entity = self.get_entity(ref)
 
-        s = re.compile("\|(\w+)\|").sub(self.substitute_property, node["arguments"][1])
+        subber = functools.partial(self.substitute_property, line=node.line)
+        s = re.compile("\|(\w+)\|").sub(subber, node["arguments"][1])
         try:
             # Attempt objectwise assignment if RHS is an entity
             bits = s.partition(".")
             donor = self.get_entity(bits[0])
             val = operator.attrgetter(bits[2])(donor.persona) if bits[2] else donor.persona
-            self.shots[-1].items.append(Model.Property(self.speaker, entity.persona, attr, val))
+            self.shots[-1].items.append(
+                Model.Property(self.speaker, entity.persona, attr, val, self.fP, node.line)
+            )
         except AttributeError:
             pass
         else:
@@ -297,9 +352,14 @@ class Model(docutils.nodes.GenericNodeVisitor):
             val = s
 
         try:
-            self.shots[-1].items.append(Model.Property(self.speaker, entity.persona, attr, val))
+            self.shots[-1].items.append(
+                Model.Property(self.speaker, entity.persona, attr, val, self.fP, node.line)
+            )
         except AttributeError:
-            warnings.warn("Entity {0} has no persona.".format(entity))
+            warnings.warn(
+                "Line {0.parent.line}: "
+                "Entity has no persona ({1}).".format(node, entity)
+            )
 
     def visit_strong(self, node):
         text = node.astext()
@@ -313,9 +373,9 @@ class Model(docutils.nodes.GenericNodeVisitor):
             defn = self.document.substitution_defs[node.attributes["refname"]]
         except KeyError:
             self.log.warning(
-                "Bad substitution ref before line {0}: {1.rawsource}".format(
-                    node.line, node
-                )
+                "Bad substitution reference",
+                {"path": self.fP, "line_nr": node.line},
+                token=node.rawsource
             )
             raise
         for tgt in defn.children:
@@ -347,17 +407,24 @@ class Model(docutils.nodes.GenericNodeVisitor):
             ))
 
     def visit_title(self, node):
-        self.log.debug(self.section_level)
+        self.log.debug(
+            "Title level {0.section_level}".format(self),
+            {"path": self.fP, "line_nr": node.line},
+            token=node.rawsource,
+        )
         if self.scenes == [None] and self.shots == [Model.Shot(None, None, [])]:
             self.scenes.clear()
             self.shots.clear()
 
         if isinstance(node.parent, docutils.nodes.section):
             if self.section_level == 1:
-                self.scenes.append(node.parent.attributes["names"][0])
+                names = node.parent.attributes["names"] + node.parent.attributes["dupnames"]
+                self.scenes.append(names[0])
             elif self.section_level == 2:
                 names = node.parent.attributes["names"] + node.parent.attributes["dupnames"]
-                self.shots.append(Model.Shot(names[0], self.scenes[-1], []))
+                self.shots.append(
+                    Model.Shot(names[0], self.scenes[-1], [], path=self.fP, line_nr=node.parent.line)
+                )
 
 
 class SceneScript:
@@ -373,8 +440,6 @@ class SceneScript:
 
     Folder = namedtuple("Folder", ["pkg", "description", "metadata", "paths", "interludes"])
 
-    log = logging.getLogger("turberfield.dialogue.model.scenescript")
-
     settings = Values(defaults=dict(
         character_level_inline_markup=False,
         debug=False, error_encoding="utf-8",
@@ -429,16 +494,22 @@ class SceneScript:
         The method generates a sequence of
         :py:class:`~turberfield.dialogue.model.SceneScript` objects.
         """
+        log_manager = LogManager()
+        log = log_manager.clone(
+            log_manager.get_logger("main"),
+            "turberfield.dialogue.model.scenescript"
+        )
+
         for path in paths:
             try:
                 fP = pkg_resources.resource_filename(pkg, path)
             except ImportError:
-                cls.log.warning(
+                log.warning(
                     "No package called {}".format(pkg)
                 )
             else:
                 if not os.path.isfile(fP):
-                    cls.log.warning(
+                    log.warning(
                         "No script file at {}".format(os.path.join(*pkg.split(".") + [path]))
                     )
                 else:
@@ -460,6 +531,10 @@ class SceneScript:
         return doc
 
     def __init__(self, fP, metadata=None, doc=None):
+        self.log_manager = LogManager()
+        self.log = self.log_manager.get_logger(
+            "turberfield.dialogue.model.scenescript"
+        )
         self.fP = fP
         self.metadata = metadata
         self.doc = doc
@@ -492,7 +567,7 @@ class SceneScript:
         rv = OrderedDict()
         performing = defaultdict(set)
         pool = list(personae)
-        self.log.debug(pool)
+        self.log.debug(pool, {"path": self.fP})
         entities = OrderedDict([
             ("".join(entity.attributes["names"]), entity)
             for entity in sorted(
@@ -532,7 +607,8 @@ class SceneScript:
                     self.log.debug(
                         "No persona for type {0} and states {1} with {2} {3}.".format(
                             typ, states, roles, "role" if roles == 1 else "roles"
-                        )
+                        ),
+                        {"path": self.fP}
                     )
         return rv
 
@@ -549,8 +625,9 @@ class SceneScript:
             self.doc.note_citation(c)
             self.doc.note_explicit_target(c, c)
             c.persona = p
-            self.log.debug("{0} to be played by {1}".format(
-                c["names"][0].capitalize(), p)
+            self.log.debug(
+                "{0} to be played by {1}".format(c["names"][0].capitalize(), p),
+                {"path": self.fP}
             )
         return self
 
